<?php

/**
  * Homepage preferences tab for admin panel, AdminHomepage.php
  * @category admin
  *
  * @author PrestaShop <support@prestashop.com>
  * @copyright PrestaShop
  * @license http://www.opensource.org/licenses/osl-3.0.php Open-source licence 3.0
  * @version 0.8
  *
  */
  
include_once(dirname(__FILE__).'/AdminPreferences.php');

class AdminHomepage extends AdminTab
{
	public function __construct()
	{
		$this->className = 'Configuration';
		$this->table = 'configuration';
 		
 		$this->_fieldsLang = array('PS_HOMEPAGE_TITLE' => 'isGenericName', 'PS_HOMEPAGE_TEXT' => 'isCleanHtml',
		'PS_META_TITLE' => 'isGenericName', 'PS_META_DESCRIPTION' => 'isGenericName', 'PS_META_KEYWORDS' => 'isGenericName');
	
		parent::__construct();
	}
	
	public function postProcess()
	{
	 	global $currentIndex;

		if (isset($_POST['submitHomepage'.$this->table]))
		{
		 	/* Language() instance determined by default language */
			$defaultLanguage = new Language(intval(Configuration::get('PS_LANG_DEFAULT')));
			
			/* All availables languages */
			$languages = Language::getLanguages();		
			
			foreach ($this->_fieldsLang AS $fieldLang => $rule)
				foreach ($languages AS $language)
					if (Tools::getValue($fieldLang.'_'.$language['id_lang']))
						if (!Validate::$rule(Tools::getValue($fieldLang.'_'.$language['id_lang'])))
							$this->_errors[] = 'field <b>'.$fieldLang.' ('.$language['name'].' language)</b> is invalid';
			
			if ($value = Tools::getValue('PS_LOGO2_LINK'))
				if (!Validate::$rule($value))
					$this->_errors[] = Tools::displayError('2nd logo link is not a valid URL');
				else
					Configuration::updateValue('PS_LOGO2_LINK', $value);
			
			if (!sizeof($this->_errors))
			{
				foreach ($this->_fieldsLang AS $fieldLang => $rule)
					foreach ($languages AS $language)
						$configurations[$fieldLang][$language['id_lang']] = Tools::getValue($fieldLang.'_'.$language['id_lang']);
				
				foreach ($configurations AS $configuration => $values)
					Configuration::updateValue($configuration, $values, $configuration == 'PS_HOMEPAGE_TEXT');
				
				$logos = array('PS_LOGO' => _PS_IMG_DIR_.'logo.jpg', 'PS_LOGO_2' => _PS_IMG_DIR_.'logo2.jpg');
				foreach ($logos AS $logo => $fileName)
					if (isset($_FILES[$logo]['tmp_name']) AND $_FILES[$logo]['tmp_name'])
					{
						if ($error = checkImage($_FILES[$logo], 300000))
							$this->_errors[] = $error;
						elseif (!imageResize($_FILES[$logo], $fileName))
							$this->_errors[] = 'an error occured during logo copy';
					}

				if (!sizeof($this->_errors))
					Tools::redirectAdmin($currentIndex.'&conf=6');
			}
		}
	}

	public function display()
	{
		header('Cache-Control: no-cache, must-revalidate');
		header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
		$this->displayForm();
	}
	
	public function gFieldValue($id_lang, $field, $confValues)
	{
		if ($value = Tools::getValue($field.'_'.intval($id_lang)))
			return $value;
		elseif (isset($confValues[intval($id_lang)][$field]))
			return $confValues[intval($id_lang)][$field];
		else
			return false;			
	}
	
	public function displayForm()
	{
		global $currentIndex;
			
		$defaultLanguage = intval(Configuration::get('PS_LANG_DEFAULT'));
		$languages = Language::getLanguages();
		$obj = $this->loadObject(true);
		$divLangName = 'PS_HOMEPAGE_TITLEPS_HOMEPAGE_TEXTPS_META_TITLEPS_META_DESCRIPTIONPS_META_KEYWORDS';
		
		foreach ($this->_fieldsLang AS $key => $value)
			$getConf[] = $key;
		$confValues = Configuration::getMultipleInt($getConf);

		echo '
		<script type="text/javascript" src="../js/tinymce/jscripts/tiny_mce/tiny_mce.js"></script>
			<script language="javascript" type="text/javascript">
				tinyMCE.init({
					language : "';
		$iso = Language::getIsoById($defaultLanguage);
		echo ((!file_exists(dirname(__FILE__).'/../../js/tinymce/jscripts/tiny_mce/langs/'.$iso.'.js')) ? 'en' : $iso).'",
					mode : "textareas",
					theme : "advanced",
					theme_advanced_buttons1 : "bold, italic, underline, fontselect, fontsizeselect",
					theme_advanced_buttons2 : "forecolor, backcolor, separator, justifyleft, justifycenter, justifyright, justifyfull, separator, bullist, numlist, separator, undo, redo",
					theme_advanced_buttons3 : "preview, code, tablecontrols, pastetext, pasteword, selectall, link, unlink, advhr",
					paste_create_paragraphs : false,
					paste_create_linebreaks : false,
					paste_use_dialog : true,
					paste_auto_cleanup_on_paste : true,
					paste_convert_middot_lists : false,
					paste_unindented_list_class : "unindentedList",
					paste_convert_headers_to_strong : true,
					theme_advanced_toolbar_location : "top",
					theme_advanced_toolbar_align : "left",
					plugins : "advhr, advlink, cleanup, paste, preview, table",
					cleanup : true,
					extended_valid_elements : "a[name|href|target|title|onclick],img[class|src|border=0|alt|title|hspace|vspace|width|height|align|onmouseover|onmouseout|name],hr[class|width|size|noshade],font[face|size|color|style],span[class|align|style]"
				});
		</script>
		<script language="javascript">id_language = Number('.$defaultLanguage.');</script>
		<form action="'.$currentIndex.'&submitAdd'.$this->table.'=1" method="post" enctype="multipart/form-data">
			<fieldset class="width3"><legend><img src="../img/admin/home.gif" />'.$this->l('Homepage').'</legend>		
				<label>'.$this->l('Main title:').' </label>
				<div class="margin-form">';
				foreach ($languages as $language)
					echo '
					<div id="PS_HOMEPAGE_TITLE_'.$language['id_lang'].'" style="display: '.($language['id_lang'] == $defaultLanguage ? 'block' : 'none').'; float: left;">
						<input size="33" type="text" name="PS_HOMEPAGE_TITLE_'.$language['id_lang'].'" 
						value="'.$this->gFieldValue(intval($language['id_lang']), 'PS_HOMEPAGE_TITLE', $confValues).'" />
					</div>';							
		displayFlags($languages, $defaultLanguage, $divLangName, 'PS_HOMEPAGE_TITLE');
		echo '		<p class="clear">'.$this->l('Will appear along top of homepage').'</p>
				</div>
				<label>'.$this->l('Introductory text:').' </label>
				<div class="margin-form">';
				foreach ($languages as $language)
					echo '
					<div id="PS_HOMEPAGE_TEXT_'.$language['id_lang'].'" style="display: '.($language['id_lang'] == $defaultLanguage ? 'block' : 'none').'; float: left; vertical-align: top;">
						<textarea cols="60" rows="15" name="PS_HOMEPAGE_TEXT_'.$language['id_lang'].'">'.$this->gFieldValue(intval($language['id_lang']), 'PS_HOMEPAGE_TEXT', $confValues).'</textarea>
					</div>';							
		displayFlags($languages, $defaultLanguage, $divLangName, 'PS_HOMEPAGE_TEXT');
		echo '		<p class="clear">'.$this->l('Text of your choice; for example, explain your mission, feature a new product, or describe a recent event').'</p>
				</div>
				<label>'.$this->l('Meta title').' </label>
				<div class="margin-form">';
				foreach ($languages as $language)
					echo '
					<div id="PS_META_TITLE_'.$language['id_lang'].'" style="display: '.($language['id_lang'] == $defaultLanguage ? 'block' : 'none').'; float: left; vertical-align: top;">
						<input type="text" name="PS_META_TITLE_'.$language['id_lang'].'" value="'.$this->gFieldValue(intval($language['id_lang']), 'PS_META_TITLE', $confValues).'" />
					</div>';							
		displayFlags($languages, $defaultLanguage, $divLangName, 'PS_META_TITLE');
		echo '		<p class="clear">'.$this->l('Appear in the titlebar of the browser').'</p>
				</div>
				<label>'.$this->l('Meta description').' </label>
				<div class="margin-form">';
				foreach ($languages as $language)
					echo '
					<div id="PS_META_DESCRIPTION_'.$language['id_lang'].'" style="display: '.($language['id_lang'] == $defaultLanguage ? 'block' : 'none').'; float: left; vertical-align: top;">
						<input type="text" name="PS_META_DESCRIPTION_'.$language['id_lang'].'" value="'.$this->gFieldValue(intval($language['id_lang']), 'PS_META_DESCRIPTION', $confValues).'" />
					</div>';							
		displayFlags($languages, $defaultLanguage, $divLangName, 'PS_META_DESCRIPTION');
		echo '		<p class="clear">&nbsp;</p>
				</div>
				<label>'.$this->l('Meta keywords').' </label>
				<div class="margin-form">';
				foreach ($languages as $language)
					echo '
					<div id="PS_META_KEYWORDS_'.$language['id_lang'].'" style="display: '.($language['id_lang'] == $defaultLanguage ? 'block' : 'none').'; float: left; vertical-align: top;">
						<input type="text" name="PS_META_KEYWORDS_'.$language['id_lang'].'" value="'.$this->gFieldValue(intval($language['id_lang']), 'PS_META_KEYWORDS', $confValues).'" />
					</div>';							
		displayFlags($languages, $defaultLanguage, $divLangName, 'PS_META_KEYWORDS');
		echo '		<p class="clear">&nbsp;</p>
				</div>
				<label>'.$this->l('Header logo:').' </label>
				<div class="margin-form">
					'.(file_exists(_PS_IMG_DIR_.'logo.jpg') ? '<img src="../img/logo.jpg" alt="'.$this->l('Logo').'" title="'.$this->l('Logo').'" /><br />' : '').'
					<input type="file" name="PS_LOGO" />
					<p style="clear: both">'.$this->l('Will appear on page and e-mail headers').'<br />(230px * 75px '.$this->l('approximately').')</p>
				</div>
				<label>'.$this->l('Homepage logo:').' </label>
				<div class="margin-form">
					'.(file_exists(_PS_IMG_DIR_.'logo2.jpg') ? '<img src="../img/logo2.jpg" alt="'.$this->l('Logo').'" title="'.$this->l('Logo').'" /><br />' : '').'
					<input type="file" name="PS_LOGO_2" />
					<p style="clear: both">'.$this->l('Will appear next to the Introductory Text above').'</p>
				</div>
				<label>'.$this->l('Homepage logo link:').' </label>
				<div class="margin-form">
					<input type="text" name="PS_LOGO2_LINK" value="'.(($value = Tools::getValue('PS_LOGO2_LINK')) ? $value : Configuration::get('PS_LOGO2_LINK')).'" />
					<p style="clear: both">'.$this->l('Link put on the 2nd logo').'</p>
				</div>
				<div class="margin-form">
					<input type="submit" value="'.$this->l('   Save   ').'" name="submitHomepage'.$this->table.'" class="button" />
				</div>
			</fieldset>
		</form>';
	}
}

?>
