<?php

/**
  * Image settings tab for admin panel, AdminImages.php
  * @category admin
  *
  * @author PrestaShop <support@prestashop.com>
  * @copyright PrestaShop
  * @license http://www.opensource.org/licenses/osl-3.0.php Open-source licence 3.0
  * @version 0.8
  *
  */
  
class AdminImages extends AdminTab
{
	public function __construct()
	{

	 	$this->table = 'image_type';
	 	$this->className = 'ImageType';
	 	$this->lang = false;
	 	$this->edit = true;
		
		$this->fieldsDisplay = array(
		'id_image_type' => array('title' => $this->l('ID'), 'align' => 'center', 'width' => 25),
		'name' => array('title' => $this->l('Name'), 'width' => 140, 'size' => 16),
		'width' => array('title' => $this->l('Width'), 'align' => 'right', 'suffix' => ' px', 'width' => 50, 'size' => 5), 
		'height' => array('title' => $this->l('Height'), 'align' => 'right', 'suffix' => ' px', 'width' => 50, 'size' => 5));
	
		parent::__construct();
	}
	
	public function displayList()
	{
		parent::displayList();
		$this->displayRegenerate();
	}
	
	public function postProcess()
	{
		global $currentIndex;
	
		if (isset($_POST['submitRegenerate'.$this->table]))
		{
			$this->_regenerateThumbnails();
			Tools::redirectAdmin($currentIndex.'&conf=9');
		}
		else
			parent::postProcess();
	}
	
	protected function _childValidation()
	{
		if (!Tools::getValue('id_image_type') AND Validate::isImageTypeName($typeName = Tools::getValue('name')) AND ImageType::typeAlreadyExists($typeName))
			$this->_errors[] = Tools::displayError('this name already exists');
	}
	
	public function displayForm()
	{
		global $currentIndex;
		
		$obj = $this->loadObject(true);

		echo $obj->id ? $this->displayWarning($this->l('After modification, do not forget to regenerate thumbnails')) : '';

		echo '
		<form action="'.$currentIndex.'&submitAdd'.$this->table.'=1" method="post">
		'.($obj->id ? '<input type="hidden" name="id_'.$this->table.'" value="'.$obj->id.'" />' : '').'
			<fieldset><legend><img src="../img/admin/picture.gif" />'.$this->l('Images').'</legend><br />
				<label>'.$this->l('Type name:').' </label>
				<div class="margin-form">
					<input type="text" name="name" value="'.$this->getFieldValue($obj, 'name').'" /> <sup>*</sup>
					<p style="clear: both;">'.$this->l('Letters only (e.g., small, medium, large, extra-large)').'</p>
				</div>
				<label>'.$this->l('Width:').' </label>
				<div class="margin-form">
					<input type="text" size="4" maxlength="5" name="width" value="'.$this->getFieldValue($obj, 'width').'" /> <sup>*</sup>
					<p style="clear: both;">'.$this->l('Maximum image width in pixels').'</p>
				</div>
				<label>'.$this->l('Height:').' </label>
				<div class="margin-form">
					<input type="text" size="4" maxlength="5" name="height" value="'.$this->getFieldValue($obj, 'height').'" /> <sup>*</sup>
					<p style="clear: both;">'.$this->l('Maximum image height in pixels').'</p>
				</div>
				<label>'.$this->l('Products:').' </label>
				<div class="margin-form">
					<input type="radio" name="products" id=products_on" value="1" '.($this->getFieldValue($obj, 'products') ? 'checked="checked" ' : '').'/> 
					<label class="t" for="products_on"><img src="../img/admin/enabled.gif" alt="'.$this->l('Enabled').'" title="'.$this->l('Enabled').'" /></label>
					<input type="radio" name="products" id="products_off" value="0" '.(!$this->getFieldValue($obj, 'products') ? 'checked="checked" ' : '').'/> 
					<label class="t" for="products_off"><img src="../img/admin/disabled.gif" alt="'.$this->l('Disabled').'" title="'.$this->l('Disabled').'" /></label>
					<p>'.$this->l('This type will be applied to products images').'</p>
				</div>
				<label>'.$this->l('Categories:').' </label>
				<div class="margin-form">
					<input type="radio" name="categories" id=categories_on" value="1" '.($this->getFieldValue($obj, 'categories') ? 'checked="checked" ' : '').'/> 
					<label class="t" for="categories_on"><img src="../img/admin/enabled.gif" alt="'.$this->l('Enabled').'" title="'.$this->l('Enabled').'" /></label>
					<input type="radio" name="categories" id="categories_off" value="0" '.(!$this->getFieldValue($obj, 'categories') ? 'checked="checked" ' : '').'/> 
					<label class="t" for="categories_off"><img src="../img/admin/disabled.gif" alt="'.$this->l('Disabled').'" title="'.$this->l('Disabled').'" /></label>
					<p>'.$this->l('This type will be applied to categories images').'</p>
				</div>
				<div class="margin-form">
					<input type="submit" value="'.$this->l('   Save   ').'" name="submitAdd'.$this->table.'" class="button" />
				</div>
				<div class="small"><sup>*</sup> '.$this->l('Required fields').'</div>
			</fieldset>
		</form>';
	}
	
	/**
	  * Display form for thumbnails regeneration
	  *
	  * @global string $currentIndex Current URL in order to keep current Tab
	  */
	public function displayRegenerate()
	{
	 	global $currentIndex;

		echo '
		<h2 class="space">'.$this->l('Regenerate thumbnails').'</h2>
		'.$this->l('Regenerates thumbnails for all existing product images').'.<br /><br />';		
		$this->displayWarning($this->l('Please be patient, as this can take several minutes'));		
		echo '
		<form action="'.$currentIndex.'" method="post">
			<input type="Submit" name="submitRegenerate'.$this->table.'" value="'.$this->l('Regenerate thumbnails').'" class="button space" onclick="return confirm(\''.$this->l('Are you sure?').'\');" />
		</form>';
	}
	
	/**
	  * Delete resized image then regenerate new one with updated settings
	  */
	private function _regenerateThumbnails()
	{	
		$productsTypes = ImageType::getImagesTypes('products');
		$categoriesTypes = ImageType::getImagesTypes('categories');

		/* Delete products images */
		$toDel = scandir(_PS_PROD_IMG_DIR_);
		foreach ($toDel AS $d)
			if (ereg('^[0-9]\-[0-9]\-(.*)\.jpg$', $d))
				unlink(_PS_PROD_IMG_DIR_.$d);		
		$productsImages = Image::getAllImages();
		
		/* Regenerate products images */
		foreach ($productsImages AS $k => $image)
		{
			if (file_exists(_PS_PROD_IMG_DIR_.$image['id_product'].'-'.$image['id_image'].'.jpg'))
				foreach ($productsTypes AS $k => $imageType)
				{
					$file['tmp_name'] = _PS_PROD_IMG_DIR_.$image['id_product'].'-'.$image['id_image'].'.jpg';
					$file['type'] = 'image/jpg';
					imageResize($file, _PS_PROD_IMG_DIR_.$image['id_product'].'-'.$image['id_image'].'-'.stripslashes($imageType['name']).'.jpg', 
					intval($imageType['width']), intval($imageType['height']));
				}
		}
		
		/* Delete categories images */
		$toDel = scandir(_PS_CAT_IMG_DIR_);
		foreach ($toDel AS $d)
			if (ereg('^[0-9]\-(.*)\.jpg$', $d))
				unlink(_PS_CAT_IMG_DIR_.$d);
			elseif (ereg('^default-(.*)\.jpg$', $d))
				unlink(_PS_CAT_IMG_DIR_.$d);
				
		/* Regenerate categories images */
		$categoriesImages = scandir(_PS_CAT_IMG_DIR_);
		foreach ($categoriesImages as $image)
			foreach ($categoriesTypes AS $k => $imageType)
			{
				$file['tmp_name'] = _PS_CAT_IMG_DIR_.$image;
				$file['type'] = 'image/jpg';
				imageResize($file, _PS_CAT_IMG_DIR_.substr($image, 0, -4).'-'.stripslashes($imageType['name']).'.jpg', 
				intval($imageType['width']), intval($imageType['height']));
			}
	}
}

?>